function iBadChans=nt_badChannels(x,threshRel,threshAbs,prop)
%iBadChans=nt_badChannels(x,threshRel,threshAbs,prop) - find bad channels using heuristics
%
%  iBadChans: bad channel numbers
%
%  x: data
%  threshRel: threshold relative to robust std over all channels [default: 3]
%  threshAbs: absolute threshold
%  prop: proportion beyond which channel is irrecuperable [default 0.7]
%
% NoiseTools
nt_greetings;

if nargin<1; error('!'); end
if nargin<2||isempty(threshRel); threshRel=3; end
if nargin<3; threshAbs=[]; end
if nargin<4; prop=0.7; end

x=nt_unfold(x);

% robust STD
w=ones(size(x));
for iIter=1:4
    xx=nt_demean(x,w);
    STD=sqrt(nt_wmean(xx(:).^2,w(:)));
    w(find(abs(xx)/STD>3))=0;
end

x=nt_demean(x,w);

iBadChans=[];
for iChan=1:size(x,2)
    if numel(find(abs(x(:,iChan))/STD>threshRel)) / size(x,1) > prop
        iBadChans=[iBadChans,iChan];
    end
end

if ~isempty(threshAbs)
    for iChan=1:size(x,2)
        if numel(find(abs(x(:,iChan))>threshAbs)) / size(x,1) > prop
            iBadChans=[iBadChans,iChan];
        end
    end
end
   
    