function [topcs,pwr,y]=nt_pca_big(x,N,chunksize,normflag)
%[topcs,pwr,y]=nt_pca_big(x,N,chunksize,normflag) - approximate first PCs of big data
%
%  r: matrix to project to PCs
%  pwr: power per PC
%  y: PCs
%
%  x: data
%  N: desired number of components [default: 10]
%  chunksize: number of channels per chunk
%  normflag: normalize at each level [default: don't]
%


if nargin<1; error('!'); end
if nargin<2||isempty(N); 
    N=10;
    disp('defaulting to N=10');
end
if nargin<3||isempty(chunksize); chunksize=500; end
if nargin<4||isempty(normflag); normflag=0; end
if chunksize<2; error('!'); end

[nsamples,nchans,ntrials]=size(x); 
x=nt_unfold(x);

if normflag;
    tonorm=1./sqrt(mean(x.^2)); 
    tonorm(find(isnan(tonorm)))=0;
    x=bsxfun(@times,x,tonorm);
end

if nchans>chunksize; 
    topcs=zeros(nchans,N*2);
    a=round(nchans/2);
    [topcs(1:a,1:N),p,z]=nt_pca_big(x(:,1:a),N,chunksize,normflag);
    [topcs(a+1:end,N+1:2*N),p,z]=nt_pca_big(x(:,a+1:end),N,chunksize,normflag);
    [topcs2,pwr,z]=nt_pca_big(x*topcs,N,chunksize,normflag);
	topcs=topcs*topcs2;
    %figure(1); plot(xx*tp(:,1)); drawnow; pause
else
    [topcs,pwr]=nt_pca0(x);
end
if size(topcs,2)>=N; 
    topcs=topcs(:,1:N); % truncate to save space
    pwr=pwr(1:N);
else
    topcs(:,end:N)=0; % should rarely happen
    pwr(end:N)=0;
end
   
%disp([size(topcs)])

if normflag; topcs=bsxfun(@times,topcs,tonorm'); end

if nargout>1; y=x*topcs; y=nt_fold(y,nsamples);end

if ~nargout % don't return, just plot something
    y=nt_unfold(y);
    semilogy(mean(y.^2), '.-');
    xlabel('pc'); ylabel('power');
    figure(2); clf
    plot(y); xlabel('sample');
    figure(3); clf
    nt_imagescc(nt_normcol(x'*y)'); 
    ylabel('pc'); xlabel('channel');
    clear topcs y
end






%% test code
if 0
    % basic
    x=randn(10000,300);
    [topcs,pwr,y]=nt_pca_big(x);
    figure(1); clf; plot(pwr);
end
if 0 
    x=sin(2*pi*3*(1:1000)'/1000)*randn(1,1000);
    x=2*x + randn(size(x));
     [topcs,pwr,y]=nt_pca_big(x);
    figure(1); plot(pwr);
    figure(2); subplot 121; plot(x); subplot 122; plot(x*topcs);
end   
if 0 
    nchans=100000;
    nnoise=900;
    x=repmat(sin(2*pi*3*(1:1000)'/1000), 1, nchans);
	x= 0.06*nt_normcol(x) + nt_normcol(randn(size(x,1),nnoise)*randn(nnoise,nchans));
    [topcs,pwr,y]=nt_pca_big(x,10);
    figure(1); plot(pwr);
    figure(2); subplot 121; plot(x(:,1:100)); subplot 122; plot(y(:,1));
end   

